/*
 *	make [-f makefile] [-ins] [target(s) ...]
 *
 *	(Better than EON mk but not quite as good as UNIX make)
 *
 *	-f makefile name
 *	-i ignore exit status
 *	-n Pretend to make
 *	-p Print all macros & targets
 *	-q Question up-to-dateness of target.  Return exit status 1 if not
 *	-r Don't not use inbuilt rules
 *	-s Make silently
 *	-t Touch files instead of making them
 *	-m Change memory requirements (EON only)
 */

#include <stdio.h>
#include "h.h"

#ifdef unix
#include <sys/errno.h>
#endif
#ifdef eon
#include <sys/err.h>
#endif
#ifdef os9
#include <errno.h>
#endif
#ifdef amiga
#include <errno.h>
#endif

#ifdef eon
#define MEMSPACE	(16384)
#endif


char	       *myname;
char	       *makefile;	/* The make file  */
#ifdef eon
unsigned	memspace = MEMSPACE;
#endif

FILE	       *ifd;		/* Input file desciptor  */
bool		domake = TRUE;	/* Go through the motions option  */
bool		ignore = FALSE; /* Ignore exit status option  */
bool		silent = FALSE; /* Silent option  */
bool		print = FALSE;	/* Print debuging information  */
bool		rules = TRUE;	/* Use inbuilt rules  */
bool		dotouch = FALSE;/* Touch files instead of making  */
bool		quest = FALSE;	/* Question up-to-dateness of file  */

static void
append_flag(flag)
char flag;
{
 /* append a flag to MAKEFLAGS and MFLAGS */

 if (strlen(getmacro("MFLAGS"))==0) {
   /* add a dash as the first character in the macro */
   appendmacro("MFLAGS", "-", FALSE);
 }

 appendmacro("MAKEFLAGS", &flag, FALSE);
 appendmacro("MFLAGS", &flag, FALSE);
}

void
main(argc, argv)
    register int    argc;
    register char **argv;
{
    register char  *p;		/* For argument processing  */
    int 	    estat = 0;	/* For question  */
    register struct name *np;
    void	    prt(), circh();
    short	    argn;

    myname = (argc-- < 1) ? "make" : *argv++;

    /* part of standard recursive make -n support */
    appendmacro("MAKE", myname, FALSE);

    /* scan the arguments and pick the option arguments */
    for (argn = 0; argn < argc; ++argn) {

	/* Now processing this one  */
	p = argv[argn];

	/* See if the argument is an option, if not continue with
	 * the next argument.
	 */
	if (*p != '-')
	    continue;

	/* It is an option. Put a NUL character in the beginning,
	 * so we don't try to process the argument more than once.
	 */
	*p = '\0';

	/* loop for all option character set in this argument */
	while (*++p != '\0') {

	    /* Add the flag to MAKEFLAGS and MFLAGS. Don't worry about
	     * if the flag is OK or not. If it is not OK, the whole
	     * job is aborted through usage().
	     */
	    append_flag(*p);

	    switch (*p) {
	    case 'f':           /* Alternate file name  */
		if (*++p == '\0') {
		    if (++argn == argc)
			usage();
		    p = argv[argn];

		    /* mark the argument as consumed */
		    argv[argn][0] = '\0';
		}
		makefile = p;
		goto end_of_args;
#ifdef eon
	    case 'm':           /* Change space requirements  */
		if (*++p == '\0') {
		    if (++argn == argc)
			usage();
		    p = argv[argn];

		    /* mark the argument as consumed */
		    argv[argn][0] = '\0';
		}
		memspace = atoi(p);
		goto end_of_args;
#endif
	    case 'n':           /* Pretend mode  */
		domake = FALSE;
		break;
	    case 'i':           /* Ignore fault mode  */
		ignore = TRUE;
		break;
	    case 's':           /* Silent about commands  */
		silent = TRUE;
		break;
	    case 'p':
		print = TRUE;
		break;
	    case 'r':
		rules = FALSE;
		break;
	    case 't':
		dotouch = TRUE;
		break;
	    case 'q':
		quest = TRUE;
		break;
	    default:		/* Wrong option  */
		usage();
	    }
	}
end_of_args:;
    }

    /* This is a rather kludgy way, but it works and I do not
     * intend to spend the rest of my life hacking pdmake. :-)
     *   The correct way would of course be to set the
     * MAKEFLAGS environment variable in the subshell and read
     * the MAKEFLAGS environment as part of the options
     * processing (before parsing the command line).
     */
    appendmacro("MAKE", getmacro("MFLAGS"), TRUE);


#if defined(amiga)
    if ((ifd = fopen("s:builtins.make", "r")) != (FILE *) 0) {
	input(ifd);
	fclose(ifd);
	firstname = (void *) NULL;
    }
    else
	makerules();
#else
    makerules();
#endif

#if defined(eon)
    if (initalloc(memspace) == 0xffff)	/* Must get memory for alloc  */
	fatal("Cannot initalloc memory");
#endif

    if (! makefile) {	/* If no file, then use default */
	if ((ifd = fopen(DEFN1, "r")) == (FILE *) 0)
#if defined(eon)
	    if (errno != ER_NOTF)
		fatal("Can't open %s; error %02x", DEFN1, errno);
#else
#  if defined(unix) || defined(amiga)
	    if (errno != ENOENT)
	        fatal("Can't open %s; error %02x", DEFN1, errno);
#  endif
#endif
#if defined(DEFN2)
	if ((ifd == (FILE *) 0)
	    && ((ifd = fopen(DEFN2, "r")) == (FILE *) 0))
	    fatal("Can't open %s", DEFN2);
#else
	/* -- hu: do nothing so we may process command line later */
#endif
    } else if (strcmp(makefile, "-") == 0)      /* Can use stdin as makefile  */
	ifd = stdin;
    else if ((ifd = fopen(makefile, "r")) == (FILE *) 0)
	; /* -- hu: again, no makefile is fine here */

    /* -- hu: if valid makefile exists, read it in now */
    if (ifd != (FILE *)0) {
	input(ifd); 		/* Input all the gunga	*/
	fclose(ifd);		/* Finished with makefile  */
    }
    lineno = 0; 		/* Any calls to error now print no line
				 * number */

    setmacro ("$", "$");

    /* Un*x make allows mixing macro definitions and targets on
     * the command line. We do the same.
     *   Here we iterate over the arguments, if we find a macro
     * definition, assign it and then put a NUL character as the
     * first character in the argument. NULs are then checked for
     * below so we do not try to make those. -- vinsci@soft.fi
     */
    for (argn = 0; argn < argc; ++argn) {
        if ((argv[argn][0]!='\0') && (p = index(argv[argn], '='))) {

	    /* append the definition to MAKE */
	    appendmacro("MAKE", argv[argn], TRUE);

	    /* do the macro definition */
	    assign_macro(argv[argn]);

	    /* NUL the argument so we don't try to make it later */
	    argv[argn][0] = '\0';
	}
    }

    if (print)
	prt();			/* Print out structures  */

    np = newname(".SILENT");
    if (np->n_flag & N_TARG)
	silent = TRUE;

    np = newname(".IGNORE");
    if (np->n_flag & N_TARG)
	ignore = TRUE;

    precious();

    circh();  /* Check circular target definitions */

    {
      bool made_at_least_one = FALSE;

      /* treat command line args as targets to make */
      while (argc--) {
	/* skip macro definition arguments we cleared out earlier */
	if (**argv == '\0')
	  ++argv;
	else {
	  if (!print && !silent && strcmp(*argv, "love") == 0) {
	    if (strcmp(myname, "make") == 0) /* -- hu: humor them */
	      printf("With me?\n");
	    else
              printf("Not War!\n");
	  }

	  np = newname(*argv++);
	  estat |= make(np, 0);
          made_at_least_one = TRUE;
	}
      }
      if (!made_at_least_one) {
	if (firstname)
	  estat = make(firstname, 0);
	else
	  fatal("No targets.");
      }
    }
    if (quest)
	exit (estat);
    else
	exit (0);
}


usage()
{
    fprintf(stderr,
"Usage: %s [-f makefile] [-inpqrst] [target(s) ...] [macro=val ...]\n", myname);
    fprintf(stderr,"\n");
    fprintf(stderr,"  -f file  Use `file' instead of `M/makefile'\n");
    fprintf(stderr,"  -i  Ignore invoked command errors (continue regardless)\n");
    fprintf(stderr,"  -n  No make (just print actions to be taken)\n");
    fprintf(stderr,"  -p  Print all macros & targets\n");
    fprintf(stderr,"  -q  Check if target is up-to-date.  Exit 1 if not\n");
    fprintf(stderr,"  -r  Don't use built-in rules\n");
    fprintf(stderr,"  -s  Make silently\n");
    fprintf(stderr,"  -t  Touch needed files instead of making them\n");
#ifdef eon
    fprintf(stderr,"  -m  Change memory requirements (EON only)\n");
#endif
    exit(1);
}


void
fatal(msg, a1, a2, a3, a4, a5, a6)
    char	   *msg;
{
    fprintf(stderr, "%s: ", myname);
    fprintf(stderr, msg, a1, a2, a3, a4, a5, a6);
    fputc('\n', stderr);
    exit(1);
}

#ifdef amiga	/*OIS*0.80*/

/*
 * The Amiga case-insensitive filing system needs a case-insensitive
 * filename comparison. Since strcmp() in this make is only applied to
 * filenames, we simply replace it.
 * Unfortunately, this also affects .PRECIOUS, .IGNORE, .SUFFIXES and
 * .SILENT. Therefore, we need a hack, but that costs space, so we don't.
 */

int lower(c)
register unsigned int c;
{
    if (c >= 'A' && c <= 'Z')
	return c - 'A' + 'a';

    return c;
}

int stricmp(first, second)	/* Case-insensitive strcmp() */
register unsigned char *first, *second;
{
    register int cmp;

    while (!(cmp = lower(*first++) - lower(*second++))) {
	if (!first[-1]) return 0;
    }

    return (cmp < 0) ? -1 : 1;
}

#ifndef LATTICE
/* Manx and PDC allow you to leave out Workbench code */

_wb_parse(){}
#endif
#endif
